//************************************************************************
// Datei: savebmp.c
//
// Speichert den Grafikschirm eines T6963 Displays als Bitmap-Datei
// auf eine Speicherkarte.
//
// Die Bitmap kann mit jedem Grafikprogramm angezeigt werden.
// Hoffe ich jedenfalls ;)
//
// Es wird immer ein komplettes Display geschrieben. Wer nur Teile
// des Displays speichern mchte muss das hier SELBST ndern.
//
// Die Routinen sind fr ein 240x64 Display ausgelegt.
// 128x64 habe ich nie probiert. Das wird mit dem 6 Bit Modus wohl noch
// ziemlich buggy sein. Im 8 Bit Modus sollte es aber keine Probleme geben.
// 
// Mit ein paar kleinen Datenpuffern sollte es mglich sein die
// Schreibgeschwindigkeit zu erhhen. Mal sehen.
//
// hk@holger-klabunde.de
// http://www.holger-klabunde.de
// 11.02.2007
// Compiler AVR-GCC 3.4.6
//************************************************************************



#include "param.h"
#include "fontsoftape.h"

//*****INCLUD-FILES FOR SD-CARD*********
#include "dos.h"			
#include "fat.h"			
#include "dosdefs.h"						
#include "mmc_spi.h"
#include "find_x.h"		
//#include "savebmp.h"
//*****INCLUD-FILES FOR SD-CARD*********

#include "lcd.h"
#include <avr/pgmspace.h>

#include "bitmap.h"


//#include "../dos.h"
//#include "../printf.h"
//#include "../serial.h"

//#############################################################
// Display saving routine. 
// Returns 0 if failed, >0 if successfull
//
// This routine only saves the graphic screen ! Not the text
// AND graphic screen you might see on your display. If you
// want to save some text also, use a graphic text font.
//
// Diese Routine speichert nur den Grafikschirm ! Nicht die
// Mischung aus Text und Grafik die man mglicherweise auf dem
// Display sieht. Wenn Text mit gespeichert werden soll, muss
// der im Grafikmodus auf das Display geschrieben werden.
//
/*{unsigned char SaveGraphDisplay(char *name, unsigned short baseadr)
//#############################################################


 unsigned char by,by1,result, fillbytes;
 unsigned char i , j;
 struct bmpheader bmp;
 
 if(baseadr!=G_BASE && baseadr!=G_BASE1) return 0;

 Remove(name); // overwrite if file exists
 
 if(Fopen(name,F_WRITE)!=F_OK) return 0;

 // Problem ! BMP's have to use 4 Byte boundarys.
 // So if you have 240 pixels and 8 pixel per byte, this is
 // 30 bytes per row. But it has to be 32 for Win !
 // For 240 pixels and 6 pixel per byte this is 40. OK.
 
 fillbytes= 4 - ( (XSIZE1/8) % 4);

 bmp.b='B';
 bmp.m='M';
 bmp.reserved=0;      //Keine Funktion
 bmp.offset=sizeof(struct bmpheader);        //Offset vom Anfang der Datei zu den Bitmapdaten
 bmp.headersize=40;    //Gre des Bitmap Headers
 bmp.width=XSIZE1;         //Breite der Bitmap in Pixeln
 bmp.height=YSIZE1;        //Hhe der Bitmap in Pixeln
 bmp.planes=1;       //Bitmap Planes
 bmp.bitsperpixel=1; //Bits pro Pixel                   
 bmp.compression=0;   //Kompression
 bmp.bitmapsize=(XSIZE1/8 + fillbytes) * YSIZE1;    //Gre der Bitmap in Bytes. Aufgerundet auf 4Byte Grenze
 bmp.hresolution=0;   //Horizontale Auflsung in Pixel/Meter
 bmp.vresolution=0;   //Vertikale Auflsung in Pixel/Meter
 bmp.colors=2;        //Anzahl Farben
 bmp.impcolors=2;//0;     //Anzahl der wichtigen Farben
 bmp.rgb0=0x00;//0x00FFFFFF; //Wei
 bmp.rgb1=0xFF;//0;          //Schwarz


 bmp.filesize=sizeof(struct bmpheader)+bmp.bitmapsize;      //Gre der gesamten Datei

// printf("fillbytes %d\n",(int)fillbytes);
// printf("bitmapsize %ld\n",bmp.bitmapsize);
// printf("headersize %d\n",(int)sizeof(struct bmpheader));


 result=Fwrite((unsigned char *)&bmp,sizeof(struct bmpheader));
 if(result != sizeof(struct bmpheader))
  {
   Fclose();
   return 0;
  }
  
 for(i=YSIZE1; i>0 ; i--)
  {

 Fclose();
 return result;

}
}
*/
unsigned char SaveGraphDisplay(char *name,unsigned const char *bitmap, unsigned short baseadr)//, unsigned short baseadr)
//#############################################################
{
 unsigned char by,by1,result, fillbytes;
 unsigned char i , j;
 struct bmpheader bmp;
 unsigned char buf[BYTES_PER_ROW+1];
 
if(baseadr!=G_BASE && baseadr!=G_BASE1) return 0;

 Remove(name); // overwrite if file exists
 
 if(Fopen(name,F_WRITE)!=F_OK) return 0;

 // Problem ! BMP's have to use 4 Byte boundarys.
 // So if you have 240 pixels and 8 pixel per byte, this is
 // 30 bytes per row. But it has to be 32 for Win !
 // For 240 pixels and 6 pixel per byte this is 40. OK.
 
 fillbytes= 4 - ( (LCD_WIDTH/8) % 4);

 bmp.b='B';
 bmp.m='M';
 bmp.reserved=0;      //Keine Funktion
 bmp.offset=sizeof(struct bmpheader);        //Offset vom Anfang der Datei zu den Bitmapdaten
 bmp.headersize=40;    //Gre des Bitmap Headers
 bmp.width=LCD_WIDTH;         //Breite der Bitmap in Pixeln
 bmp.height=LCD_HEIGHT;        //Hhe der Bitmap in Pixeln
 bmp.planes=1;       //Bitmap Planes
 bmp.bitsperpixel=1; //Bits pro Pixel                   
 bmp.compression=0;   //Kompression
 bmp.bitmapsize=(LCD_WIDTH/8 + fillbytes) * LCD_HEIGHT;    //Gre der Bitmap in Bytes. Aufgerundet auf 4Byte Grenze
 bmp.hresolution=0;   //Horizontale Auflsung in Pixel/Meter
 bmp.vresolution=0;   //Vertikale Auflsung in Pixel/Meter
 bmp.colors=2;        //Anzahl Farben
 bmp.impcolors=0;     //Anzahl der wichtigen Farben
 bmp.rgb0=0x00FFFFFF; //Wei
 bmp.rgb1=0;          //Schwarz

 bmp.filesize=sizeof(struct bmpheader)+bmp.bitmapsize;      //Gre der gesamten Datei

// printf("fillbytes %d\n",(int)fillbytes);
// printf("bitmapsize %ld\n",bmp.bitmapsize);
// printf("headersize %d\n",(int)sizeof(struct bmpheader));


 result=Fwrite((unsigned char *)&bmp,sizeof(struct bmpheader));
 if(result != sizeof(struct bmpheader))
  {
   Fclose();
   return 0;
  }
  
 for(i=0; i<LCD_WIDTH; i++)
  {
   // BMP's want to be drawn from down to up !
  // WriteData2(baseadr + ((i-1) * BYTES_PER_ROW ) ,0x24); //Setze Startadresse des zu lesenden Bereiches
 

#ifdef PIXPERBYTE_8    // than also 8 pixel per byte for graphic
   for(j=0; j < BYTES_PER_ROW; j++)
    {
    // WriteCommand(0xC1); //data read and increment ADP
    // by=ReadData();
    
     by=pgm_read_byte_far(   0x10000UL + (uint32_t) (uint16_t)  bitmap);
	 
      buf[j]=by;
    } //for j
 
   result=Fwrite(&by,BYTES_PER_ROW);
   bitmap++;
#endif   
    
   for(j=0; j<fillbytes; j++) buf[j]=0;

   result=Fwrite(buf,(unsigned int)fillbytes);
   
  }// for i

 Fclose();
 return result;
}



//************************************************************************************************
//************************************************************************************************










//##########################################################
//##########################################################
//##########################################################
//##########################################################
//##########################################################
unsigned char SaveGraphDisplay1(char *name,unsigned const char *bitmap)
//##########################################################
{


unsigned int written;
 unsigned char i,j,k, result;
 unsigned short by,mask,bwidth,yp,xp;
// unsigned char by,mask;
 unsigned int xend;     //xpos + width kann grer als 255 werden

 xend=(unsigned short)0 + (unsigned short)80;
 if(xend>XSIZE) xend=XSIZE; //Bitmap abschneiden wenn auerhalb des Displays
 
 bwidth=80/8; //Anzahl Bytes horizontal /8
 if((80%8)!=0) bwidth++; //Bei Rest noch ein Byte mehr




if(GetDriveInformation()!=F_OK) // get drive parameters
  {
   // Gib hier eine Fehlermeldung raus
     LCDSoftText("KEINE SD-CARD GEFUNDEN",1,1); 
  
  }
           if(Fopen(name,'a')!=F_OK) return 0;
           {


 yp=0;
 for(i=0; i<63; i++) //ber die gesamte Hhe
  {
   xp=0; //Zum Beginn der Zeile
   
   for(j=0; j<bwidth; j++) //ber die gesamte Breite in Bytes
    {
	//lsst sich nur der Flash-Speicher zwischen 0k.. 64k adressieren.
     //by=pgm_read_byte(bitmap);

//by=pgm_read_byte_far(FAR(MyBitmap));



    by='\pgm_read_byte(bitmap)';
//result=Fwrite(by,1);
	//lsst sich nur der Flash-Speicher zwischen 64k..128k adressieren.
	// by=pgm_read_byte_far(0x10000UL + (uint32_t) (uint16_t) bitmap);
	 mask=0x80;         //Fngt bei MSB an
    
     for(k=0; k<8; k++) //ber ein Byte
      {
        //Nur vorhandene und keine Fllpixel,-bytes zeichnen
       if(xp < (unsigned short)xend)
        {
         //lcd_setpixel(xp,yp,(by & mask) );
         //   written=Fwrite(by & mask,1);
	
result=Fwrite(by,1);
result=Fwrite(&by,1);
	//    by='\%';
   //result=Fwrite(&by,1);
		 xp++; 
        }                               

       mask>>=1; //Nchstes Bit
      }//for k

     bitmap++;
    } //for j

   yp++; //Nchste Zeile
 	    by='\E';
   result=Fwrite(&by,1);
   Fclose(); //Datei schliessen
   return result;  
   
  } //for i
}
}























